#!/usr/bin/env sh

# ----------------------------------------------------------- #
# Copyright (C) 2008 Red Hat, Inc.                            #
# Written by Michel Samia <msamia@redhat.com>                 #
# aliases.sh                                                  #
# more info in aliases.dsc                                    #
# ----------------------------------------------------------- #


. "${TESTS_DIRS}/bash_defs.sh"

W_COMMAND_NOT_FOUND=1
W_INCLUDED_FILE_NOT_FOUND=2
E_GROUP_WRITABLE=3
E_WORLD_WRITABLE=4
E_NOT_OWNED_BY_ROOT=5

# test all defined aliases
bash -l -c 'alias' | sed 's/^alias//' |
{
	while read line
	do
		name="`echo $line | awk -F= '{ print $1 }'`"
		value="`echo $line | awk -F= '{ print $2 }' | sed "s/^'//" | sed "s/'$//"`" # all what is ritht from '=' , without border apostrophs

		# todo: check all commands, since one alias can contain more, separated i.e. by ';', '&&' and so on

		#value="`"echo $value" | sed 
		
		# take a first command..
		command="`echo $value | awk '{ print $1 }' `"
		if [[ "$command" == "." ]]
		then
			included_file="`echo $value | awk '{ print $2 }' `"
			is_command="0"
		else
			is_command="1"
		fi

		if [[ "$is_command" == "1" ]]
		then
			fullpath="`which $command 2>&1`"
			commandWithoutPath="`basename $command`"	

			# does is exist?
			if [[ "`echo "$fullpath" |grep -c "no $commandWithoutPath in"`" == "1" ]]
			then
				if [[ "$command" != "alias" ]] # fedora uses something strange for 'which'..
				then
					report 'WARNING' $W_COMMAND_NOT_FOUND "Alias '$name' contains command '$command', which was not found"
				fi
			else

				# is this file secure? (means not world/group writable + owned by root
				perm="`stat -L -c '%A' $fullpath`"
				if [[ ${perm:5:1} != '-' ]]
				then
					report 'ERROR' $E_GROUP_WRITABLE "Alias '$name' contains command '$command', which is is group writable"
					report 'HINT' $E_GROUP_WRITABLE "Please unset it the group writing bit"
				fi

				
				if [[ ${perm:8:1} != '-' ]]
				then
					report 'ERROR' $E_WORLD_WRITABLE "Alias '$name' contains command '$command', which is is world writable"
					report 'HINT' $E_WORLD_WRITABLE "Please unset it the world writing bit"
				fi
				
				owner="`stat -c '%U' $fullpath`"
				if [[ "$owner" != "root" ]]
				then
					report 'ERROR' $E_NOT_OWNED_BY_ROOT "Alias '$name' contains command '$command', which is not owned by root"
					report 'HINT' $E_NOT_OWNED_BY_ROOT "Please chown this file to root:root"
				fi
			fi
	
		fi
		
		# it is an included file
		if [[ "$is_command" == "0" ]]
		then
			# does it exist? 
			if ! [[ -f $included_file ]]
			then
				report 'WARNING' $W_INCLUDED_FILE_NOT_FOUND "Alias '$name' includes file '$included_file', which does not exist"
			else
			
				# is this file secure? (means not world/group writable + owned by root
				perm="`stat -L -c '%A' $included_file`"
				if [[ ${perm:5:1} != '-' ]]
				then
					report 'ERROR' $E_GROUP_WRITABLE "Alias '$name' includes file '$included_file', which is is group writable"
					report 'HINT' $E_GROUP_WRITABLE "Please unset it the group writing bit"
				fi

				
				if [[ ${perm:8:1} != '-' ]]
				then
					report 'ERROR' $E_WORLD_WRITABLE "Alias '$name' includes file '$included_file', which is is world writable"
					report 'HINT' $E_WORLD_WRITABLE "Please unset it the world writing bit"
				fi
				
				owner="`stat -c '%U' $fullpath`"
				if [[ "$owner" != "root" ]]
				then
					report 'ERROR' $E_NOT_OWNED_BY_ROOT "Alias '$name' includes file '$included_file', which is not owned by root"
					report 'HINT' $E_NOT_OWNED_BY_ROOT "Please chown this file to root:root"
				fi
			fi
		fi
	done
}
