#!/usr/bin/env bash

. "${TESTS_DIRS}/bash_defs.sh"

const ID_CRONTABPERM=1
const ID_CRONDIRPERM=2
const ID_DIFFCRONSHELL=3
const ID_MISSINGPATHDIR=4
const ID_UNEXPECTEDDIR=5
const ID_CRONWRONGOWNER=6

const SEDCMD="$(which sed)"
const STATCMD="$(which stat)"
const WHICHCMD="$(which which)"
const GREPCMD="$(which grep)"

# Check crontab perm
check_file_perm "${CRONTAB}" "${CRONTAB_PERM}" "${CRONTAB_OWNER}" 1 $ID_CRONTABPERM "tables for driving cron"

# Check crondirs perm
while read dir; do
    check_file_perm "${dir}" "${CRONDIRS_PERM}" "${CRONDIRS_OWNER}" 1 $ID_CRONDIRPERM "cron directory"
done <<EOF
${CRONDIRS}
EOF

var NEWPATH

# Check vars in crontab
while read vars; do
    eval "${vars}"

    case "${VNAME}" in
	SHELL)
	    if [[ "${VAR_SHELL}" != "${CRON_SHELL}" ]]; then
		report 'WARNING' $ID_DIFFCRONSHELL "crontab variable SHELL is set to \"${VAR_SHELL}\". Expected value is \"${CRON_SHELL}\"."
	    fi
	    ;;
	PATH)
	    TMP=$IFS
	    IFS=$'\n'
	    EXP_DIRS=($((sort | uniq) <<< "${CRON_PATH_DIRS}"))
	    CUR_DIRS=($((tr ':' '\n' | sort | uniq) <<< "${VAR_PATH}"))
	    IFS=$TMP

	    for ((i=0; i < ${#EXP_DIRS[*]}; i++)); do
		if [[ -z "${CUR_DIRS[i]}" ]]; then
		    report 'WARNING' $ID_MISSINGPATHDIR "Missing directory \"${EXP_DIRS[i]}\" in PATH."
		    break
		fi

		if [[ "${CUR_DIRS[i]}" != "${EXP_DIRS[i]}" ]]; then
		    if (( ${#CUR_DIRS[*]} < ${#EXP_DIRS[*]} )); then
		    	report 'WARNING' $ID_MISSINGPATHDIR "Missing directory \"${EXP_DIRS[i]}\" in PATH."
		    else
		    	report 'ERROR' $ID_UNEXPECTEDDIR "Unexpected directory \"${CUR_DIRS[i]}\" found in PATH. Expected is \"${EXP_DIRS[i]}\"."
		    fi
		    break;
		fi
	    done

	    NEWPATH="${VAR_PATH}"
	    ;;
    esac
done <<EOF
$(sed -n "s|^[[:space:]]*\([A-Z]*\)=\(.*\)[[:space:]]*$|VNAME=\"\1\" VAR_\1='\2'|p" "${CRONTAB}")
EOF

PATH="${NEWPATH}"

# Check crontab records
while read vars; do
    eval "${vars}"

    case "${CMD}" in
	/*)
	    CMDPATH="${CMD}"
	    ;;
	*)
	    CMDPATH="$(${WHICHCMD} "${CMD}")"
	    ;;
    esac

    CMDOWNER="$(${STATCMD} -c "%U:%G" "${CMDPATH}")"

    if ! (echo "${CMDOWNER}" | ${GREPCMD} -q "${CRON_CMD_OWNER}"); then
	report 'WARNING' $ID_CRONWRONGOWNER "Wrong owner/group \"${CMDOWNER}\" on \"${CMDPATH}\". Expected is \"${CRON_CMD_OWNER}\"."
    fi
done <<EOF
$(${SEDCMD} -n -e "s|^\([0-9\*,-]\+\)[[:space:]]\+\([0-9\*,-]\+\)[[:space:]]\+\([0-9\*,-]\+\)[[:space:]]\+\([0-9\*,-]\+\)[[:space:]]\+\([0-9\*,-]\+\)[[:space:]]\+\([a-zA-Z0-9_-]*\)[[:space:]]\+\([^;`\$[:space:]]\+\)[[:space:]]*\(.*\)$|MIN=\"\1\" HRS=\"\2\" DAYM=\"\3\" DAYW=\"\4\" MON=\"\5\" USER=\"\6\" CMD='\7' CMDARGS='\8'|p" "${CRONTAB}")
EOF

test_exit ${E_OK}
