#!/usr/bin/env bash
#
# Copyright (C) 2008 Red Hat, Inc.
# Written by Daniel Kopecek <dkopecek@redhat.com>

. "${TESTS_DIRS}/bash_defs.sh"

const ID_NOHOMEDIR=1
const ID_DIFFOWNER=2
const ID_DIRWORLDWR=3
const ID_DIRWORLDRD=4
const ID_DIRGROUPWR=5
const ID_EMPTYSHELL=6
const ID_MOREROOTUIDS=7
const ID_NEWDIRINROOT=8
const ID_ATTRDIRCHANGE=9
const ID_FIRSTRUN=10
const ID_UNKNOWNDIR=11
const ID_DIFFOWNERGID=12
const ID_DIRWORLDEX=13
const ID_DIRWORLDEX2=14

var UID_ZERO_LIST=""
int UID_ZERO_CNT=0
array __USER_ARRAY=($SYSTEM_ACCOUNTS)
array __USER_DIRS
array __HOME_IGNORE=($HOME_DIRS_IGNORE)

function is_system_acct () {
    local name="${1}"
    typeset -i w=${#__USER_ARRAY[*]}
    typeset -i s=0
    typeset -i h
    
    while ((w > 0)); do
	((h=s+w/2))
	if [[ "${name}" == "${__USER_ARRAY[h]}" ]]; then
            return $TRUE
	elif [[ "${name}" > "${__USER_ARRAY[h]}" ]]; then
            ((s= h + 1))
            ((w= w - w/2 - 1))
            continue
	else
            ((w=w/2))
	fi
    done

    return $FALSE
}

function is_known_dir () {
    local name="${1}"
    typeset -i w=${#__USER_DIRS[*]}
    typeset -i s=0
    typeset -i h
    
    while ((w > 0)); do
	((h=s+w/2))
	if [[ "${name}" == "${__USER_DIRS[h]}" ]]; then
            return $TRUE
	elif [[ "${name}" > "${__USER_DIRS[h]}" ]]; then
            ((s= h + 1))
            ((w= w - w/2 - 1))
            continue
	else
            ((w=w/2))
	fi
    done

    return $FALSE
}

function ignore_inhome () {
    local name="${1}"
    typeset -i w=${#__HOME_IGNORE[*]}
    typeset -i s=0
    typeset -i h
    
    while ((w > 0)); do
	((h=s+w/2))
	if [[ "${name}" == "${__HOME_IGNORE[h]}" ]]; then
            return $TRUE
	elif [[ "${name}" > "${__HOME_IGNORE[h]}" ]]; then
            ((s= h + 1))
            ((w= w - w/2 - 1))
            continue
	else
            ((w=w/2))
	fi
    done

    return $FALSE
}

function add_user_dir () {
    __USER_DIRS[${#__USER_DIRS[*]}]="${1}"
}

########################################################

if [[ ! -f "${TDATA_DIR}/${HOME_DIRS_FILE}" ]]; then
    report 'WARNING' $ID_FIRSTRUN "This is a first run of the test. Some parts of audit are skipped."

    report 'INFO' "Creating list of home directories and their attributes."
    while read dir; do
	stat -c "MODE: %a UID: %u GID: %g   ${dir}" "${dir}" >> "${TDATA_DIR}/${HOME_DIRS_FILE}"
    done <<EOF
`find ${HOME_ROOT_DIRS} -maxdepth 1 -mindepth 1 | sort`
EOF
fi

while read line; do
    eval "$(sed -n "s|^\([^\`\$:;]\+\):[^\`$:;]\+:\([0-9]\+\):\([0-9]\+\):[^\`$:;]*:\([^\`$:;]\+\):\([^\`$:;]*\)$|user=\"\1\" uid=\2 gid=\3 home=\"\4\" shell=\"\5\"|p" <<<"${line}")"
    
    if (( uid == 0 )); then
	((++UID_ZERO_CNT))

	if (( UID_ZERO_CNT > 1)); then
	    UID_ZERO_LIST="${UID_ZERO_LIST}, ${user}"
	else
	    UID_ZERO_LIST="${user}"
	fi
    fi
    
    add_user_dir "${home}"
    
    if [[ "${shell}" != "/sbin/nologin" && "${shell}" != "/usr/sbin/nologin" ]]; then
	if [[ -n "${shell}" ]]; then
	    if ! is_system_acct "${user}"; then
		report 'INFO' "Checking home directory of user \"${user}\""

		stat="$(stat -c 'perm="%a" owner_uid="%u" owner_gid="%g"' "${home}")"
		
		if (( $? != 0 )); then
		    if [[ "${home}" != "/nonexistent" ]]; then
			report 'ERROR' $ID_NOHOMEDIR "Home directory of user \"${user}\" does not exist!"
		    fi
		else
		    eval "${stat}"

		    if (( owner_uid != uid )); then
			report 'ERROR' $ID_DIFFOWNER "Home directory of user \"${user}\" has wrong uid: ${owner_uid}. Expected uid is ${uid}."
		    fi

		    if (( owner_gid != gid )); then
			report 'WARNING' $ID_DIFFOWNERGID "Home directory of user \"${user}\" has wrong gid: ${owner_gid}. Expected gid is ${gid}."
		    fi

		    if (( 8#$perm & 8#00002 )); then
			report 'ERROR' $ID_DIRWORLDWR "Home directory of user \"${user}\" is world-writable."
		    fi

		    if (( 8#$perm & 8#00004 )); then
			report 'ERROR' $ID_DIRWORLDRD "Home directory of user \"${user}\" is world-readable."
		    fi

		    if (( 8#$perm & 8#00001 )); then
		    	if (( 8#$perm & 8#00004 )); then
		    		report 'ERROR' $ID_DIRWORLDEX2 "Home directory of user \"${user}\" is world-accessible."
		    	else
		    		report 'WARNING' $ID_DIRWORLDEX "Home directory of user \"${user}\" is world-accessible."
		    	fi
		    fi

		    if (( 8#$perm & 8#00020 )); then
			report 'WARNING' $ID_DIRGROUPWR "Home directory of user \"${user}\" is group-writable."
		    fi
		fi
	    fi
	else
	    report 'ERROR' $ID_EMPTYSHELL "Shell of user \"${user}\" is NOT set!"
	fi
    fi
done <<EOF
`getent passwd`
EOF

# Sort user dirs
TMP=$IFS
IFS=$'\n'
__USER_DIRS=($(echo -e "${__USER_DIRS[*]}" | sort | uniq))
IFS=$TMP

if (( UID_ZERO_CNT > 1)); then
    report 'ERROR' $ID_MOREROOTUIDS "More that one user with uid 0: ${UID_ZERO_LIST}"
fi

if [[ -f "${TDATA_DIR}/${HOME_DIRS_FILE}" ]]; then
    while read dir; do
	OUTPUT1="$(stat -c "MODE: %a UID: %u GID: %g   ${dir}" "${dir}")"
	
	if (( $? != 0 )); then
	    test_exit ${E_FAIL} "stat -f \"MODE:%6p UID:%6u GID: %g   ${dir}\" \"${dir}\" failed."
	fi
	
	OUTPUT2="$(grep " ${dir}" "${TDATA_DIR}/${HOME_DIRS_FILE}")"
	
	if [[ -z "${OUTPUT2}" ]]; then
	    BASE="$(dirname "${dir}")"
	    report 'WARNING' $ID_NEWDIRINROOT "New file or directory in ${BASE}: ${dir}"
	else
	    if [[ "${OUTPUT2}" != "${OUTPUT1}" ]]; then
		ORIG="$(cut -d ' ' -f '-6' <<<"${OUTPUT1}")"
		CUR="$(cut -d ' ' -f '-6' <<<"${OUTPUT2}")"
		
		report 'WARNING' $ID_ATTRDIRCHANGE "Attributes of \"${dir}\" changed: ${ORIG} -> ${CUR}."
	    fi
	fi
	
	if ! (is_known_dir "${dir}" || ignore_inhome "$(basename "${dir}")"); then
	    if [[ -d "${dir}" ]]; then
		report 'WARNING' $ID_UNKNOWNDIR "Directory \"${dir}\" doesn't belong to anybody."
	    fi
	fi
    done <<EOF
`find ${HOME_ROOT_DIRS} -maxdepth 1 -mindepth 1 | sort`
EOF
fi

test_exit ${E_OK}
