#!/usr/bin/env sh
. "${TESTS_DIRS}/bash_defs.sh"
# -------------------------------- #
# Copyright (C) 2008 Red Hat, Inc.
# Written by Dan Kopecek <dkopecek@redhat.com>

# --- Error IDs ------- #
const ID_CMDSHADOWED=1
const ID_DIRNOTPRESENT=2
const ID_VERIFYFAILED=3
const ID_FILENOTFOUND=4

function check_dirs () {
    local RET=${E_OK}

    while read dir; do
	if [[ ! -d "${dir}" ]]; then
	    report 'ERROR' $ID_DIRNOTPRESENT "Directory \"${dir}\" does not exist!"
	    RET=${E_FATAL}
	fi
    done <<EOF
${DIRECTORY_LIST}
EOF

    return ${RET}
}

function check_cmds () {
    local RET=${E_OK}
    
    OUTPUT="$(${RPMVERIFY} --noscript --nomtime -f "${COREPKG}" 2> /dev/null)"
    if (( $? != 0 )); then
	if [[ -n "$(echo -e "${OUTPUT}" | sed -n 's|^[.A-Z0-9]\{8\}[[:space:]]\{1,\}[^c][[:space:]]\{1,\}.*$|&|p')" ]]; then
	    report 'ERROR' $ID_VERIFYFAILED "Verify failed for package \"${COREPKG}\"!"
	    test_exit ${E_FATAL} "integrity test failed."
	fi
    fi
       
    while read path; do    
    	if [[ "$(stat -c '%A' "${path}" | cut -c 1)" != "l" ]]; then 
	    COMMANDNAME="$(basename "${path}")"
	    COMMANDPATH="$(dirname  "${path}")"
	    COMMANDPATHS="$(echo "${CMDPATH}" | tr ':' ' ' | sed -e "s|[[:space:]]${COMMANDPATH}||g" -e "s|^${COMMANDPATH}[[:space:]]||g")"
	    
	    unset DUPLICATES
	    I=0
	    
	    while read duplicate; do
		if [[ -n "${duplicate}" ]]; then
	    	    if [[ "$(stat -c '%A' "${duplicate}" | cut -c 1)" != "l" ]]; then
	    		DUPLICATES[$I]="${duplicate}"
	    		I=$(($I + 1))
	    	    else
	    		DEST="$(readlink -e "${duplicate}" 2> /dev/null)"
	    		if [[ "${DEST}" != "${path}" ]]; then
	    		    DUPLICATES[$I]="${duplicate}"
	    		    I=$(($I + 1))
	    		fi	
	    	    fi
		fi
	    done <<EOF
`find ${COMMANDPATHS} -maxdepth 1 -mindepth 1 -name "${COMMANDNAME}" 2> /dev/null`
EOF
	    if (( $I > 0 )); then
		report 'ERROR' $ID_CMDSHADOWED "Command \"${COMMANDNAME}\" (${path}) has duplicates in these locations: ${DUPLICATES[*]}!"
		RET=${E_FATAL}
	    fi
	fi
    done <<EOF
`rpm -ql "${COREPKG}" | grep bin/`
EOF

    return ${RET}
}

# --- Main ----------------------- #

PATH="${CMDPATH}"

check_dirs || (test $? == ${E_FATAL} && test_exit ${E_FATAL} "Integrity test failed")
check_cmds || (test $? == ${E_FATAL} && test_exit ${E_FATAL} "Integrity test failed")

test_exit ${E_OK}
# --- EOF ------------------------ #
