#!/usr/bin/env python
#coding=utf-8

from python_defs import *

MOUNT_POINT = 1
FS_TYPE     = 2
OPTS        = 3

i = 1

local_types = get_var('LOCAL_TYPES').split()
removable_types = get_var('REMOVABLE_TYPES').split()
remote_types = get_var('REMOTE_TYPES').split()
files = get_var('FILES')
blacklist = get_var('BLACKLIST').split()

remote_required = get_var('REMOTE_REQ').split()
remote_suggested = get_var('REMOTE_SUGG').split()
removable_required = get_var('REMOVABLE_REQ').split()
local_required = get_var('LOCAL_REQ').split()

def expand_defaults(options):
    if 'defaults' in options:
        options.remove('defaults')
        options.extend(get_var('DEFAULTS'))
    return options


def check_remote_opts(file, linenum, line_list):
    global i
    options = line_list[OPTS].split(',')
    options = expand_defaults(options)

    for opt in remote_required:
        if opt not in options:
            report(WARNING, i, 'The mountpoint %s doesn\'t have "%s" option set.' % (line_list[MOUNT_POINT], opt)) 
            report(HINT, i, 'Edit %s and add "%s" to the fourth field of line %d' % (file, opt, linenum))

    wrong = []
    for one_opt in remote_suggested:
        if one_opt not in options:
            wrong.append(one_opt)

    if len(wrong) > 0:
        opt_str = ','.join(wrong)
        report(WARNING, i, 'The mountpoint %s doesn\'t have "%s" option(s) set. ' % (line_list[MOUNT_POINT], opt_str)) 
        report(HINT, i, 'This is false warning if there must be executables on the share (like nfs-mounted /usr)')

def check_removable_opts(file, linenum, line_list):
    global i
    options = line_list[OPTS].split(',')
    options = expand_defaults(options)

    if 'defaults' in options:
        options.remove('defaults')
        options.extend(get_var('DEFAULTS'))

    wrong = []
    for one_opt in removable_required:
        if one_opt not in options:
            wrong.append(one_opt)


    if len(wrong) > 0:
        opt_str = ','.join(wrong)
        report(WARNING, i, 'The mountpoint %s doesn\'t have "%s" option(s) set. ' % (line_list[MOUNT_POINT], opt_str)) 
        report(HINT, i, 'Edit %s and add "%s" to the fourth field of line %d' % (file, opt_str, linenum))

def check_local_opts(file, linenum, line_list): 
    global i
    options = line_list[OPTS].split(',')
    options = expand_defaults(options)

    if 'defaults' in options:
        options.remove('defaults')
        options.extend(get_var('DEFAULTS'))

    for opt in local_required:
        if line_list[MOUNT_POINT] != '/' and opt not in options:
            report(WARNING, i, 'The local mountpoint %s is not / but doesn\'t have "%s" option set. ' % (line_list[MOUNT_POINT], opt)) 
            report(HINT, i, 'This is false warning if the system uses chroot jails.')
            i = i + 1


def main():
    for file in files.split():
        file_content = open(file).readlines()
        for index, line in enumerate(file_content):
            if line.startswith('#') or line.strip() == '':
                continue

            # don't check blacklisted mountpoints
            splitted = line.split()
            if splitted[MOUNT_POINT] in blacklist:
                continue

            if splitted[FS_TYPE] in removable_types:
                check_removable_opts(file, index+1, splitted)
            if splitted[FS_TYPE] in remote_types:
                check_remote_opts(file, index+1, splitted)
            if splitted[FS_TYPE] in local_types:
                check_local_opts(file, index+1, splitted)

if __name__ == "__main__":
    main()
