#!/usr/bin/env sh

# ----------------------------------------------------------- #
# Copyright (C) 2008 Red Hat, Inc.                            #
# Written by Michel Samia <msamia@redhat.com>                 #
# path.sh                                                     #
# secTool script for testing directories in $PATH             #
# more info in path.dsc                                       #
# ----------------------------------------------------------- #



. "${TESTS_DIRS}/bash_defs.sh"


const E_BAD_PATH_ELEM=1
const W_NONEXISTING_DIR=2
const E_WORLD_WRITABLE=3
const E_GROUP_WRITABLE=4
const E_WRITABLE_FILE=5
const E_OWNER_NOT_ROOT=6

#
# a little workaround to get real PATH, not the one got by userhelper
# 

REALPATH="`bash -cl 'echo $PATH'`"


what="::"
report 'DEBUG' $E_BAD_PATH_ELEM "Testing whether directory \$what is in variable PATH" '1'

## Tests whether the element :: is in $PATH. This empty element works the same as ./
## in bash
if [[ `echo $REALPATH | egrep $what | wc -l` -eq  "1" ]]
then
    report 'ERROR' $E_BAD_PATH_ELEM "Element \$what found in variable PATH!"
    report 'HINT' $E_BAD_PATH_ELEM "Element \$what allows running binaries from the current directory without specifying full path"
else
    report 'INFO' "Test whether directory \$what is in the PATH finished with success"
fi

what="./"
report 'DEBUG' $E_BAD_PATH_ELEM "Testing whether directory \$what is in variable PATH" '1'

# . or ./ on the first place of the $PATH | in the middle of the $PATH | at the end of $PATH
# for testing purposes you can use $1 instead of $PATH ;-)
if [[ `echo $REALPATH | egrep '^\./?:|:\./?:|:./?$' | wc -l` -eq  "1" ]] 
then
    report 'ERROR' $E_BAD_PATH_ELEM "Directory \$what found in variable PATH!"
    report 'HINT' $E_BAD_PATH_ELEM "Element \$what allows running binaries from the current directory without specifying full path"
else
    report 'INFO' "Test whether directory \$what is in the PATH finished with success"
fi


# /tmp in path
what="/tmp"
report 'DEBUG' $E_BAD_PATH_ELEM "Testing whether directory \$what is in variable PATH" '1'

if [[ `echo $REALPATH | egrep '^/tmp/?:|:/tmp/?:|:/tmp/?$' |wc -l` -eq "1" ]]
then
    report 'ERROR' $E_BAD_PATH_ELEM "Directory \$what found in variable PATH!"
    report 'HINT' $E_BAD_PATH_ELEM "Direcotry \$what shouldn't be specified in PATH as it's usually world-writable"
else
    report 'INFO' "Test whether directory \$what is in the PATH finished with success"
fi



# test of presence and write permission of all dirs in PATH
echo "$REALPATH:" |
{
    while read -d: dir
    do
	# does the directory exist?
	if ! [[ -d $dir ]]
	then
	    report 'WARNING' $W_NONEXISTING_DIR "Path variable contains directory \$dir, which doesn't exist"
	    report 'HINT' $W_NONEXISTING_DIR "Please look in your startup scripts and remove this setting"
	else	
		perm="`stat -c '%A' $dir`"
		if [[ "${perm:5:1}" == "w" ]]
		then
		    report 'ERROR' $E_GROUP_WRITABLE "Path variable contains directory \$dir, which is group-writable"
		    report 'HINT' $E_GROUP_WRITABLE "Please look in your startup scripts and remove this setting. This directory can be used to make you execute a fake (and dangerous) binary file, i.e. ls"
		fi

		if [[ "${perm:8:1}" == "w" ]]
		then
		    report 'ERROR' $E_WORLD_WRITABLE "Path variable contains directory \$dir, which is world-writable"
		    report 'HINT' $E_WORLD_WRITABLE "Please look in your startup scripts and remove this setting. This directory can be used to make you execute a fake (and dangerous) binary file, i.e. ls"
		fi
		
		# group or others writable executable
		find -L $dir -perm /o+w -perm /g+w |
		{
			while read file
			do
				if [[ -f "$file" ]]
				then
					report 'ERROR' $E_WRITABLE_FILE "File \${file}, which is placed in PATH directory \${dir}, is writable NOT only by his owner! The correct permissions should be 755."
					report 'HINT' $E_WRITABLE_FILE "This is VERY dangerous, since one can change contents of this executable. Please set permissions of this file to the recommended one immediately!"
				fi
			done
		}
		
		# owner other than root and can log in
		find -L $dir ! -user root |
		{
		    while read file
		    do
			user="`stat -c '%U' $file`"
			if canLogIn "$user"; then
			    report 'ERROR' $E_OWNER_NOT_ROOT "File \${file}, which is placed in PATH directory \${dir}, is owned by someone else than by root!"
			    report 'HINT' $E_OWNER_NOT_ROOT "This is VERY dangerous, since one can change contents of this executable. Please change the owner and group of this file to 'root' immediately!"
			fi
		    done
		}
	fi
    done
}
