# Helper functions for perl tests
# Lubomir Kundrak <lkundrak@redhat.com>

# Use them as follows:
# use lib '/usr/share/sectool/tests';
# use perl_defs;

# TODO: gettext

# Global functions
sub debug_msg
{
	print STDERR @_;
}

sub report
{
	my $type = shift
		or debug_msg ('Expected type');

	my %format_map = (
		WARNING	=> "W%02d : %s\n",
		ERROR	=> "E%02d : %s\n",
		HINT	=> "H%02d : %s\n",
		INFO	=> "I : %s\n", # == DEBUG
	);

	if (defined $format_map{$type}) {
		printf $format_map{$type}, @_;
	} else {
		debug_msg ("Bad type: $type");
	}
}

sub test_exit
{
	my $code = shift;
	my $test_name = shift
		or debug_msg ('Expected test name');
	my $msg = shift or '';

	if ($code == 0) {
		print "Test: $test_name, Succeeded";
		return $code;
	} else {
		print STDERR "Test: $test_name, Error: $code - $msg";
		exit $code;
	}
}

sub isset
{
	my $patam = shift
		or debug_msg ('Expected parameter');

	$param =~ /^1|yes|true$/ and return 0;
	return 1;
}

1;
