#!/usr/bin/env sh

# ----------------------------------------------------------- #
# Copyright (C) 2008 Red Hat, Inc.                            #
# Written by Michel Samia <msamia@redhat.com>                 #
# disc_usage.sh                                               #
# more info in disc_usage.dsc                                 #
# ----------------------------------------------------------- #

# secTool script testing directories from standarad hierarchy for presence, permissions and owner
# for more info about FHS see http://www.pathname.com/fhs/pub/fhs-2.3.html


. "${TESTS_DIRS}/bash_defs.sh"


E_DOESNT_EXIST=1
E_WRONG_OWNER=2
E_WRONG_PERM=3

dirs="/ 755
/bin 755
/boot 755
/dev 755
/etc 755
/home 755
/lib 755
/media 755
/mnt 755
/opt 755
/root 750
/sbin 755
/srv 755
/tmp 1777
/usr 755
  /usr/bin 755
  /usr/sbin 755
  /usr/include 755
  /usr/lib 755
  /usr/share 755
  /usr/src 755
  /usr/local 755
/var 755
  /var/lock 775
  /var/log 755
  /var/mail 777
  /var/run 755
  /var/spool 755
  /var/spool/mail 775
  /var/tmp 1777"


echo "$dirs" |
{ while read dir perm
do
    #echo "dir: $dir   perm: $perm"
        
    # exists?
    if ! [[ -d $dir ]]
    then
	report 'ERROR' $E_DOESNT_EXIST "Directory \$dir doesn't exist!"
	report 'HINT' $E_DOESNT_EXIST "Please create this directory."
    else
    
	# owner
	if [[ "`stat -c '%U' $dir`" != "root" ]]
	then
	    report 'ERROR' $E_WRONG_OWNER "Directory \$dir has wrong owner"
	    report 'HINT' $E_WRONG_OWNER "Please change owner of this directory to root."
	fi

	echo "real perm is `stat -c '%a' $dir`"
	if [[ "`stat -c '%a' $dir`" != "$perm" ]]
	then
	    report 'ERROR' $E_WRONG_PERM "Directory \$dir has wrong permissions! The correct permissions should be \$perm."
	    report 'HINT' $E_WRONG_PERM "Please change the permissions to the recommended value."
	fi
    fi
done
}
