#coding=utf-8

import sys
import os
import stat
import hashlib
import md5
import rpm
import subprocess
from os import environ as testenv

## Global constants
# return codes
E_OK=0
E_FAIL=1
E_FATAL=2

# report types
WARNING=1
ERROR=2
HINT=3
INFO=4
DEBUG=4

## Global functions
def debug_msg(msg):
    """ Emits a message to stderr. Useful for debugging. """
    print >>sys.stderr, msg

def report(type, id, text):
    """ This outputs a message 'text' in the standartized form """
    if type == WARNING:
        print "W%02d : %s\n" % (id, text)
    elif type == ERROR:
        print "E%02d : %s\n" % (id, text)
    elif type == HINT:
        print "H%02d : %s\n" % (id, text)
    elif type == INFO or type == DEBUG:
        print "I : %s\n" % (text)
    else:
        debug_msg("Bad parameter to report: %s" % (str(type)))

def test_exit(code, test_name, msg=""):
    """ Outputs a succeeded message if code == 0 or failed message if code != 0 """
    if code == 0:
        print "Test: %s, Succeeded" % (test_name)
        return E_OK
    else:
	print >>sys.stderr, "Test: %s, Error: %d - %s" % (test_name, code, msg)
        sys.exit(code)

def isset(param):
    try:
        int(param)
    except ValueError:
        if param.upper() == "YES" or param.upper == "TRUE":
            return E_OK
        if param.upper() == "NO" or param.upper == "FALSE":
            return E_FAIL
    else:
        if param == 1:
            return E_FAIL
        if param == 0:
            return E_OK
    finally:
        return E_FAIL

def check_file_perm(file, perms, user, group, verbose=False, strict=False):
    if os.path.exists(file):
        statres = os.lstat(file)

        f_perms = oct(stat.S_IMODE(statres[stat.ST_MODE]) & 0777)[1:]
        if strict:
            perms_ok = (f_perms == perms)
        else:
            perms_ok = (f_perms <= perms)
        if perms_ok == False:
            if verbose:
                report(WARNING, 1, "Bad permissions on %s: found %d, expected %d" % (file, f_perms, perms))
            return E_FAIL

        if pwd.getpwuid(statres[stat.ST_UID]) != user:
            if verbose:
                report(WARNING, 1, "Bad user on %s: found %s, expected %s" % (file, pwd.getpwuid(statres[stat.ST_UID]), user))
            return E_FAIL

        if grp.getgrgid(statres[stat.ST_GID]) != group:
            if verbose:
                report(WARNING, 1, "Bad group on %s: found %s, expected %s" % (file, grp.getgrgid(statres[stat.ST_GID]), group))
            return E_FAIL

        return E_OK
    else:
        report(ERROR, 1, "Missing file or directory %s" % (file))
        return E_FAIL

def sha256file(file_name):
    hash = hashlib.sha256()
    file = open(file_name)
    [ hash.update(l) for l in file.readlines() ]
    file.close()
    return hash.hexdigest()

def md5file(file_name):
    return md5.new(open(file_name).read()).hexdigest()

def get_var(var):
    try:
        return testenv[var]
    except KeyError:
        return None

def rpm_are_modified(file_names):
    hash = dict()

    stdout, stderr = subprocess.Popen(["/bin/rpm", "-Vf"] + file_names, stdout=subprocess.PIPE).communicate()

    for line in stdout.split('\n'):
        if line.strip() == "": continue
        fields = line.split()
        fname = fields[(len(fields)-1)]
        if fname in file_names:
            hash[fname] = fields[0]

    return hash

def rpm_md5(file_name):
    rpm_ts = rpm.TransactionSet()
    match = rpm_ts.dbMatch(rpm.RPMTAG_BASENAMES, file_name)

    file_list = []
    md5_sums  = []
    for hdr in match:
        file_list = hdr['filenames']
        md5_sums  = hdr['filemd5s']

    if(len(file_list) == 0):
        return None
    else:
        try:
            return md5_sums[file_list.index(file_name)]
        except IndexError:
            print >>sys.stderr, "Could not query md5 sum of %s from rpm database..this is probably a bug" % (file_name)
            return None

# uid2user == pwd.getpwuid()
# git2user == grp.getgrgid()
# rel2abs  == os.path.abspath()
