#!/usr/bin/env sh

# ----------------------------------------------------------- #
# Copyright (C) 2008 Red Hat, Inc.                            #
# Written by Michel Samia <msamia@redhat.com>                 #
# routing.sh                                                  #
# more info in routing.dsc                                    #
# ----------------------------------------------------------- #


. "${TESTS_DIRS}/bash_defs.sh"

ID_FIRSTRUN=1
ID_RULE_ADDED=2
ID_RULE_DELETED=3

const RTABLE="${TDATA_DIR}/routing_table"
const NEW_RTABLE="${TDATA_DIR}/routing_table.new"
const DIFF="${TDATA_DIR}/routing_table.diff"

if ! [[ -f $RTABLE ]]
then
	report 'WARNING' $ID_FIRSTRUN 'This is a first run of the test. Some parts of audit are skipped.'
#	touch "${RTABLE}" || test_exit ${E_FAIL} "Can't create a file in my permanent data directory."

	# save the current contents of routing tables
	route | tail -n +2 > ${RTABLE}
else
	route | tail -n +2 > ${NEW_RTABLE}
	diff ${RTABLE} ${NEW_RTABLE} > ${DIFF}

	# make it invisible for other users
	chmod 600 ${TDATA_DIR}/*

	if [[ "`cat $DIFF | wc -l`" == "0" ]]
	then
		exit
	fi

	# else report the differences
	egrep '^>|^<' $DIFF |
	{
		while read line
		do
			if [[ "${line:0:1}" == ">" ]]
			then
				report 'WARNING' $ID_RULE_ADDED "Routing rule '${line:2}' was added to the routing table since the last run"
			elif [[ "${line:0:1}" == "<" ]]
			then
				report 'WARNING' $ID_RULE_DELETED "Routing rule '${line:2}' was deleted from the routing table since the last run"
			fi
		done
	}
fi
