#!/usr/bin/env sh

# ----------------------------------------------------------- #
# Copyright (C) 2008 Red Hat, Inc.                            #
# Written by Michel Samia <msamia@redhat.com>                 #
# passwd.sh                                                   #
# more info in passwd.dsc                                     #
# ----------------------------------------------------------- #

# TODO:
#  

. "${TESTS_DIRS}/bash_defs.sh"

#constants
const passwd=/etc/passwd
const group=/etc/group
const shadow=/etc/shadow
const group_shadow=/etc/gshadow

# --- Error IDs ------- #

const E_BAD_PERMISSIONS_SHADOW=2
const W_EMPTY_LINE=3
const E_WRONG_NUMBER_OF_FIELDS=4
const E_NO_USERNAME=5
const E_NONALNUM_CHARS=6
const E_LONG_USERNAME=7
const E_USER_WITH_NO_PASSWORD=8
const E_UID_ZERO_LOGIN_NOT_ROOT=10
const E_UID_ONE_LOGIN_NOT_BIN=11
const E_GID_ZERO_LOGIN_NOT_ROOT=12
const E_GID_ONE_LOGIN_NOT_BIN=13
const E_NEGATIVE_UID=14
const E_NEGATIVE_GID=15
const E_SAME_UIDS=16
const E_SAME_LOGINS=17
const E_ROOT_NOT_ZERO=18





#if[[ `whoami` -ne 'root' ]]
if [[ $UID -ne '0' ]]
then
    gettext "You have to be logged as root to run this test!" 1>&2 ; echo
    exit ${E_FAIL}
fi

# permissions on /etc/shadow and /etc/gshadow should be 400, should be owned by root:root
check_file_perm $shadow 400 root:root  1 $E_BAD_PERMISSIONS_SHADOW "User shadow database"
check_file_perm $group_shadow 400 root:root  1 $E_BAD_PERMISSIONS_SHADOW "Group shadow database"


{
	i=0
	while read line
	do
		i=$[i+1]
		
		##### empty line #####
		if [[ "$line" == "" ]]
		then
			report 'WARNING' $W_EMPTY_LINE "\$shadow: Line \$i is empty"
			report 'HINT' $W_EMPTY_LINE "Please delete this line."
			continue
		fi

		##### number of fields #####
		nf=`echo "$line" | awk -F: '{printf NF}'`
		if [ "$nf" -ne "9" ]
		then
			report 'ERROR' $E_WRONG_NUMBER_OF_FIELDS "\$shadow: Line \$i has wrong number of fields"
			report 'HINT' $E_WRONG_NUMBER_OF_FIELDS "Please see 'man 5 shadow' and correct this line."
			continue
		fi	
		
		# now we can parse these fields, we know that all fields exist
		username="`echo $line | awk -F: '{print $1}'`"
		pass="`echo $line | awk -F: '{print $2}'`"
		uid="`echo $line | awk -F: '{print $3}'`"
		gid="`echo $line | awk -F: '{print $4}'`"
		
		##### line has an empty login field #####
		if [[ "$username" == "" ]]
		then
			report 'ERROR' $E_NO_USERNAME "\$shadow: Line \$i: missing username!"
			report 'HINT' $E_NO_USERNAME "Check this line, fill in first item (username), or delete whole line."
		fi
		
		##### disallowed characters #####
		isValidName $username
		if [ $? -ne 1 ]
		then
			report 'ERROR' $E_NONALNUM_CHARS "\$shadow: Line \$i: User \$username contains disallowed characters in his login."
			report 'HINT' $E_NONALNUM_CHARS "Check this line and rename user's login to contain lowercase letters only."
		fi
		
		##### too long username #####
		getValueFromH '/usr/include/bits/utmp.h' 'UT_NAMESIZE'
		MaxLength=$ReturnVal	
		length="`echo \"$username\" | wc -m`"
		if [ $length -gt $MaxLength ]
		then
			report 'ERROR' $E_LONG_USERNAME "\$shadow: Line \$i: User \$username has too long username."
			report 'HINT' $E_LONG_USERNAME "Check this line in \$shadow and rename user's login to be shorter than \$MaxLength characters"
		fi
		
		##### password empty #####
		if [[ "$pass" == "" ]]
		then
			report 'ERROR' $E_USER_WITH_NO_PASSWORD "\$shadow: Line \$i: User \$username has no password!"
			report 'HINT' $E_USER_WITH_NO_PASSWORD "Run 'passwd' utility immediately to set his password!"
		fi
		
		##### unusual password hash length #####
		#if[[  ]]
	done
} < $shadow


##### two users with same username #####
awk -F: '{ if ($1 != "") print $1 }' $shadow | sort | uniq -d |
{
    while read user
    do
	lines="`grep -n -e "^$user:" $shadow | awk -F: '{ print $1 }'| tr '\n' ','`"
	lines="${lines%','}" # remove last comma
	report 'ERROR' $E_SAME_LOGINS "Duplicate login '\$user' (\$shadow lines $lines)"
	report 'HINT' $E_SAME_LOGINS "Please change usernames on these lines to be different or delete duplicate records"
    done
}

