#!/usr/bin/python
#coding=utf-8

from python_defs import *

import sys
import stat
import os
import os.path
import commands

LIST='setuid.list'

def is_suid(file):
    try:
        return os.lstat(file)[stat.ST_MODE] & stat.S_ISUID == stat.S_ISUID
    except OSError:
        return False

def get_suid_list(dir, files):
    ret = []

    for f in files:
        file_path = os.path.join(dir, f)
        if is_suid(file_path):
            ret.append(file_path)

    return ret

def add_to_lists(file_list, files, sums):
    for f in file_list:
        files.append(f)
        sums.append(md5file(f))
    
def main():
    report(INFO, 01, 'This test passes if there are no differencies against the previous run')

    i = 2
    this_run_files     = []
    this_run_md5s      = []
    previous_run_files = []
    previous_run_md5s  = []
    first_run          = False

    data_dir = get_var('TDATA_DIR')
    if data_dir:
        if os.path.exists(os.path.join(data_dir, LIST)):
            for l in open(os.path.join(data_dir, LIST), 'r').read().splitlines():
                file, md5 = l.split()
                previous_run_files.append(file)
                previous_run_md5s.append(md5)
        else:
            first_run = True
            report(WARNING, i, 'This is a first run of the test. Some parts of audit are skipped.')
            report(HINT, i, 'Run this test periodically to get diffs of results')
    else:
        report(ERROR, i, 'This test expects a data directory which was not given, bailing out..')
        test_exit(E_ERROR, testenv['NAME'], 'No data directory given')

    path = get_var('CHECK_PATH')
    if path:
        for gen in [ os.walk(dir) for dir in testenv['PATH'].split(':') ]:
            for dir,subdirs,files in gen:
                add_to_lists(get_suid_list(dir, files), this_run_files, this_run_md5s)

    check_dir = get_var('CHECK_DIR')
    if check_dir:   
        for one_check_dir in check_dir.split():
            for dir, subdirs, files in os.walk(one_check_dir, topdown=True):
                if os.path.ismount(dir) and one_check_dir != dir:
                    del subdirs[:]
                add_to_lists(get_suid_list(dir, files), this_run_files, this_run_md5s)

    # check if all the setuid files found belong to a package
    i = i + 1
    package_cmd = get_var('PACKAGE_CMD')
    if package_cmd:
        for binary in this_run_files:
            if commands.getstatusoutput('%s %s' % (get_var('PACKAGE_CMD'), binary))[0] != 0:
                report(WARNING, i, 'The file %s doesn\'t belong to any package' % (binary)) 
                report(HINT, i, 'Check what the file does, who owns it, who created it, etc.')

    i = i + 1

    # report differencies in files found
    if first_run == False:
        this_run_set = set(this_run_files)
        previous_run_set = set(previous_run_files)
        for binary in this_run_set - previous_run_set:
            report(WARNING, i, 'The file %s is setuid now and wasn\'t in the previous run' % (binary)) 
            report(HINT, i, 'Check if it was added intentionally')

        for binary in previous_run_set - this_run_set:
            report(WARNING, i, 'The file %s was setuid in the last run and is not now' % (binary))
            report(HINT, i, 'Check if it was changed intentionally')

        # report changed md5files
        i = i + 1
        changed = rpm_are_modified(this_run_files)
        for file, verify_tags in changed.items():
            if '5' in verify_tags:
                report(ERROR, i, 'The file %s has changed its md5sum according to rpmdb' % (file))
                report(HINT, i, 'Check if it was changed intentionally')

       # We now check md5 sums via the rpmdb - uncomment this if you want to check by hand..does not work with prelink, though
       # for item in previous_run_set & this_run_set:
       #     this_md5 = this_run_md5s[this_run_files.index(item)] 
       #     previous_md5 = previous_run_md5s[previous_run_files.index(item)]
       #     if this_md5 != previous_md5:
       #         report(ERROR, i, 'The file %s has changed its md5sum from %s to %s' % (item, previous_md5, this_md5))
       #         report(HINT, i, 'Check if it was changed intentionally')

    if first_run:
        # write the list of setuids seen this time
        umask_old = os.umask(077)
        open(os.path.join(data_dir, LIST), 'w').writelines('\n'.join([ "%s %s" % (this_run_files[index], this_run_md5s[index]) for (index, f) in enumerate(this_run_files) ] ))
        os.umask(umask_old)

if __name__ == "__main__":
    main()

