#!/usr/bin/env sh

# ----------------------------------------------------------- #
# Copyright (C) 2008 Red Hat, Inc.                            #
# Written by Michel Samia <msamia@redhat.com>                 #
# tcp_wrappers.sh                                             #
# more info in tcp_wrappers.dsc                               #
# ----------------------------------------------------------- #


. "${TESTS_DIRS}/bash_defs.sh"

ALLOW='/etc/hosts.allow'
DENY='/etc/hosts.deny'

const W_NOPOLICY=1

# get list of applications, which are linked with libwrap
apps="`egrep -l libwrap /usr/sbin/* /usr/bin/* 2>/dev/null | egrep -v config | xargs -n 1 basename`"
ps="`ps -eo comm=`"

echo "$apps" |
{
	while read app
	do

		# TODO: it would be better to grep it with sth like "^[^:#]*$app|^ALL *:" to work with lists of services on one line...

		echo $ps | grep $app > /dev/null
		if [ $? -eq 1 ]
		then
			# the application is not running, so we will not cry about it..
			continue
		fi

		#report 'INFO' 1 "App: $app"

		egrep "^$app|^ALL *:" $ALLOW > /dev/null
		isInAllow=$[ 1 - $? ]

		egrep "^$app|^ALL *:" $DENY > /dev/null
		isInDeny=$[ 1 - $? ]
		
		if [ $isInAllow -ne 1 ] && [ $isInDeny -ne 1 ]
		then
			report 'WARNING' $W_NOPOLICY "Application \$app is using tcp wrappers, but does not specify any restrictions in \$ALLOW nor in \$DENY"
			report 'HINT' $W_NOPOLICY "Please consider restricting access to this service i.e. according to IP address ranges"
		fi
		
	done
}
