#!/usr/bin/env python
#coding=utf-8

from python_defs import *

import sys
import stat
import os
import os.path
import commands
import string

global config_file
global passwd_file
config_file = get_var( "CONFIG_FILE" )
passwd_file = get_var( "PASSWD_FILE" )

class Configuration:

    """
        Class used for parsing configuration file. Describes functions to save, load coonfiguration or
        restart server.
    """

    def __init__( self, path ):
        
        """
        Object for work with configuration
        """

        self.file = path

        #: Default options by Red Hat, please DO NOT CHANGE !
        self.default = { # Daemon options
                        "listen"                    : "NO",
                        "listen_ipv6"               : "NO",
                        "background"                : "NO",
                        "session_support"           : "NO",

                        # ---Log In Options and Access Controls---------------------
                        "banned_email_file"         : "/etc/vsftpd/banned_emails", 
                        "banner_file"               : "",
                        "cmds_allowed"              : "",
                        "deny_email_enable"         : "NO",
                        "ftpd_banner"               : "",
                        "pam_service_name"          : "ftp", 
                        "tcp_wrappers"              : "NO",
                        "userlist_deny"             : "YES",
                        "userlist_enable"           : "NO",
                        "userlist_file"             : "/etc/vsftpd/user_list", 
                        "cmds_allowed"              : "", 

                        # ---Anonymous Use Options----------------------------------
                        "anonymous_enable"          : "YES",
                        "anon_mkdir_write_enable"   : "NO",
                        "anon_root"                 : "",
                        "anon_upload_enable"        : "NO",
                        "anon_world_readable_only"  : "YES",
                        "ftp_username"              : "ftp", 
                        "no_anon_password"          : "NO",
                        "anon_umask"                : "077",
                        "allow_anon_ssl"            : "NO", 
                        
                        # ---Local User Options-------------------------------------
                        "local_enable"              : "NO",
                        "chmod_enable"              : "YES",
                        "chroot_list_enable"        : "NO",
                        "chroot_list_file"          : "/etc/vsftpd/chroot_list",
                        "chroot_local_user"         : "NO",
                        "guest_enable"              : "NO",
                        "guest_username"            : "ftp",
                        "local_root"                : "",
                        "passwd_chroot_enable"      : "NO",
                        "user_config_dir"           : "",


                        # ---Directory Options--------------------------------------
                        "dirlist_enable"            : "YES",
                        "dirmessage_enable"         : "NO",
                        "force_dot_files"           : "NO",
                        "hide_ids"                  : "NO",
                        "message_file"              : ".message",
                        "text_userdb_names"         : "NO",
                        "use_localtime"             : "NO",
                        "local_umask"               : "077",
                        "deny_file"                 : "", 
                        "hide_file"                 : "",
                        "secure_chroot_dir"         : "/usr/share/empty",

                        # ---File Transfer Options----------------------------------
                        "download_enable"           : "YES",
                        "chown_uploads"             : "NO",
                        "chown_username"            : "root",
                        "write_enable"              : "NO",
                        "file_open_mode"            : "0666",

                        # ---Logging Options----------------------------------------
                        "dual_log_enable"           : "NO",
                        "log_ftp_protocol"          : "NO",
                        "syslog_enable"             : "NO",
                        "vsftpd_log_file"           : "/var/log/vsftpd.log",
                        "xferlog_enable"            : "NO",
                        "xferlog_file"              : "/var/log/xferlog",
                        "xferlog_std_format"        : "NO",
                        "no_log_lock"               : "NO",

                        # ---Network Options----------------------------------------
                        "accept_timeout"            : "60",
                        "anon_max_rate"             : "0",
                        "connect_from_port_20"      : "NO",
                        "connect_timeout"           : "60",
                        "data_connection_timeout"   : "300",
                        "ftp_data_port"             : "20",
                        "idle_session_timeout"      : "300",
                        "listen_address"            : "",
                        "listen_address"            : "",
                        "listen_port"               : "21",
                        "local_max_rate"            : "0",
                        "max_clients"               : "0",
                        "max_per_ip"                : "0",
                        "pasv_address"              : "",
                        "pasv_enable"               : "YES",
                        "pasv_max_port"             : "0",
                        "pasv_min_port"             : "0",
                        "pasv_promiscuous"          : "NO",
                        "port_enable"               : "YES",
                        "trans_chunk_size"          : "0",

                        # ---SSL----------------------------------------------------
                        "ssl_enable"                : "NO",
                        "force_local_data_ssl"      : "YES",
                        "force_local_logins_ssl"    : "YES",
                        "ssl_sslv2"                 : "NO",
                        "ssl_sslv3"                 : "NO",
                        "ssl_tlsv1"                 : "YES",
                        "dsa_cert_file"             : "",  
                        "dsa_private_key_file"      : "",
                        "rsa_cert_file"             : "/usr/share/ssl/certs/vsftpd.pem",
                        "rsa_private_key_file"      : "",
                        "ssl_ciphers"               : "DES-CBC3-SHA",

                        # ---Other Options------------------------------------------
                        "anon_other_write_enable"   : "NO",
                        "ascii_download_enable"     : "NO",
                        "ascii_upload_enable"       : "NO",
                        "async_abor_enable"         : "NO",
                        "check_shell"               : "YES",
                        "ls_recurse_enable"         : "NO",
                        "one_process_model"         : "NO", # 2.4 kernel only - mention it in Ws ?
                        "run_as_launching_user"     : "NO",
                        "secure_email_list_enable"  : "NO",
                        "setproctitle_enable"       : "NO",
                        "tilde_user_enable"         : "NO",
                        "use_sendfile"              : "YES",
                        "virtual_use_local_privs"   : "NO",
                        "email_password_file"       : "/etc/vsftpd/email_passwords",
                        "nopriv_user"               : "nobody",
                        "user_sub_token"            : ""
                        }

        self.current = self.loadConfig( path )


    def __getitem__( self, item ):
        """
        Method called when variable is getting from object
        """

        if item in self.current: return self.current[item]
        elif item in self.default: return self.default[item]
        else: return None

    def loadConfig( self, fname ):
        """
        Method for loading configuration from server's configuration file (/etc/vsftpd/vsftpd.conf)
        """

        __config = {}

        #: read the file to the list of lines
        #: check first if it exists
        if not os.path.exists( fname ): 
            report(ERROR, 0, "Configuration file \'%s\' does not exist. Do not run this test if you have not installed vsftpd." % (fname) )
            report(HINT, 0, "Try to change CONFIG_FILE variable in vsftpd's dsc file.")
            test_exit( 0, get_var("NAME"), "Testing vsftpd failed." )
            sys.exit( -1 )

        confFile = open( fname, 'r')
        __lines = confFile.readlines()
        confFile.close( )

        # remove comments and free lines
        try: 
            for line in __lines:
                if line[0] != "#" and len(line) > 2:
                    line = string.rstrip( line )
                    list = string.split( line, "=" )
                    if string.upper( list[1] ) == "YES" or string.upper( list[1] ) == "NO":
                        __config[ list[0] ] = string.upper( list[1] )
                    else: 
                        __config[ list[0] ] = list[1]
        except IndexError:
            #: how we make error messages ?? TODO
            return None

        return __config

def check_read( file ):

    try:
        return os.lstat(file)[stat.ST_MODE] & stat.S_IREAD == stat.S_IREAD and \
               os.lstat(file)[stat.ST_MODE] & stat.S_IRUSR == stat.S_IRUSR
    except OSError:
        return False

def get_user_configs( config ):
    """ Get all user configs the 'user_config_dir' path """

    user_config_dir = config["user_config_dir"]
    user_configs = {}
    if user_config_dir != "" and os.path.exists( user_config_dir ):
        for item in os.listdir( user_config_dir ):
            fname = os.path.join( user_config_dir, item )
            if fname != '' and stat.S_ISREG( os.lstat( fname )[stat.ST_MODE] ) and item[0] != ".":
                user_configs[ item ] = Configuration( fname )

    return user_configs

def get_option( option, config ):

    value = None
    value = config[option]

    return value

def evaluate_options( options, config ):

    ret = True
    for option in options:
        value = get_option( option[1:], config )
        if value == None: return False
        if option[0] == '!':
            if value == "": ret = False
        elif option[0] == '+':
            if value == 'NO': ret = False
        elif value == 'YES': ret = False
        elif option[0] != '-': test_exit(E_FATAL, get_var('NAME'), "Bad option in options in function 'evaluate_options'")
    return ret

def get_ftp_home_attributes( ):

    passwdFile = open( passwd_file, 'r')
    lines = passwdFile.readlines()
    passwdFile.close( )

    dir = None
    uid = None
    for line in lines:
        list = line.split( ":" )
        if list[0] == "ftp": 
            dir = list[5]
            uid = list[2]

    return [dir, uid]



def main():
    """ Main function """

    config = Configuration( config_file )
    # we have main configuration file, we need all configuration files !
    user_configs = get_user_configs( config )
    #: now we have all configs and all options for all user, great ! Now lets do some real work.
    repno = 0

    #: checking permissions on the vsftpd.conf file
    repno += 1
    #report(INFO, repno, "Testing permissions of config file \'%s\'..." % (config.file) )
    stat_res = os.lstat( config.file )
    f_perms = oct(stat.S_IMODE(stat_res[stat.ST_MODE]) & 0777)[1:]
    if f_perms > oct(0700)[1:]: #: maybe group of users could have permissions to access this file.. for now we don't want that
        report(ERROR, repno, "Config file %s has wrong permissions: \'0%s\' !" % (config.file, f_perms))
        report(HINT, repno, "Run command \'chmod 0600 %s\'" % (config.file) )

    #: check if the server is listening and if it's listening only on one protocol (IPv4 or IPv6)
    repno += 1
    if get_option( "listen", config ) == "YES" and get_option( "listen_ipv6", config ) == "YES":
        report(WARNING, repno, "You are trying to start server on both protocols (IPv4 and IPv6) at once. ")
        report(HINT, repno, "You have to start two instances of the server" )
    repno += 1
    if get_option( "listen", config ) == "NO"  and get_option( "listen_ipv6", config ) == "NO":
        report(INFO, repno, "Server is not listening in standalone mode. You have to start it from inetd on some kind.")

    #: checking all files if present
    #: if there is '+' before option, it is neccessary to have YES otherwise NO
    #: if there is prefix '!', option has to be set, otherwise do not check 

    test_files = [[['+deny_email_enable'],['banned_email_file']],
                  [['+userlist_enable'],['userlist_file']],
                  [['+chroot_list_enable'],['chroot_list_file']],
                  [['+secure_email_list_enable'],['email_password_file']],
                  [['!banner_file'],['banner_file']],
                  [['!dsa_cert_file'],['dsa_cert_file']],
                  [['!dsa_private_key_file'],['dsa_private_key_file']],
                  [['!rsa_cert_file','+ssl_enable'],['rsa_cert_file']],
                  [['!rsa_private_key_file'],['rsa_private_key_file']],
                  [['!local_root'],['local_root']],
                  [['!anon_root'],['anon_root']],
                  [['!user_config_dir'],['user_config_dir']],
                 ]

    #: check all options if the file is neccessary 
    for options in test_files:
        if evaluate_options( options[0], config ):
            #report(INFO, repno, "Testing for options \'%s\'..." % (options[0]) )
            for option in options[1]:
                file = get_option( option, config )
                #: we need the file.. check if it exists ...
                if not os.path.exists( file ):
                    report( ERROR, repno+1, 'File \'%s\' does not exist! (%s option)' % (file, option) )
                #: ... and check if its readable
                elif not check_read( file ):
                    report( ERROR, repno+2, 'File \'%s\' is not readable, please check permissions. (%s option)' %(file, option) )
    
    log_files = [[['+xferlog_enable','-xferlog_std_format'],['vsftpd_log_file']],
                 [['+xferlog_enable','+xferlog_std_format'],['xferlog_file']],
                 [['+dual_log_enable'],['vsftpd_log_file','xferlog_file']],
                ]
    #: check log files
    for options in log_files:
        if evaluate_options( options[0], config ):
            for option in options[1]:
                file = get_option( option, config )
                #: we need the file.. check if it exists ...
                if not os.path.exists( file ):
                    report( WARNING, repno+1, 'File \'%s\' does not exist. (%s option)' % (file, option) )
                #: ... and check if its readable
                elif not check_read( file ):
                    report( ERROR, repno+2, 'File \'%s\' is not readable, please check permissions. (%s option)' %(file, option) )

    user_files = [[['+deny_email_enable'],['banned_email_file']],
                  [['!banner_file'],['banner_file']],
                  [['!dsa_cert_file'],['dsa_cert_file']],
                  [['!dsa_private_key_file'],['dsa_private_key_file']],
                  [['!rsa_cert_file','+ssl_enable'],['rsa_cert_file']],
                  [['!rsa_private_key_file'],['rsa_private_key_file']],
                  [['!local_root'],['local_root']],
                  [['!anon_root'],['anon_root']],
                  [['!user_config_dir'],['user_config_dir']],
                 ]

    #: check local_root for every user's config
    path = get_option( 'user_config_dir', config )
    for options in user_files:
        for user in user_configs.keys():
            if evaluate_options( options[0], user_configs[user] ):
                for option in options[1]:
                    file = get_option( option, user_configs[user] )
                    #: we need the file.. check if it exists ...
                    if not os.path.exists( file ):
                        report( ERROR, repno+1, 'File \'%s\' does not exist ! (%s option in \'%s\')' % (file, option, os.path.join( path, user)) )
                    #: ... and check if its readable
                    elif not check_read( file ):
                        report( ERROR, repno+2, 'File \'%s\' is not readable, please check permissions. (%s option)' %(file, option) )

    repno += 2

    #: check permissions of anonymous directory
    repno += 1
    if evaluate_options( ['+anonymous_enable','!anon_root'], config ):
        ftp_user = get_ftp_home_attributes( )
        file = get_option( 'anon_root', config )
        if file != '': ftp_user[0] = file

        file_stat = os.lstat( ftp_user[0] )
        if ``file_stat[stat.ST_UID]``.strip() == `ftp_user[1]`.strip() and file_stat[stat.ST_MODE] & stat.S_IWRITE == stat.S_IWRITE:
            report(WARNING, repno, "You have writable anonymous home directory \'%s\', it's danger !" % (ftp_user[0]))
            report(HINT, repno, "The home directory should NOT be owned by the ftp user itself. \\nWay to fix this: \\nchown root %s \\nchmod -w %s" % (ftp_user[0], ftp_user[0]))
        elif file_stat[stat.ST_MODE] & stat.S_IWOTH == stat.S_IWOTH:
            report(WARNING, repno, "You have writable anonymous home directory \'%s\' by other users, it's danger !" % (ftp_user[0]))
            report(HINT, repno, "The home directory should NOT be owned by the ftp user itself. \\nWay to fix this: \\nchown root %s \\nchmod -w %s" % (ftp_user[0], ftp_user[0]))

        

    #: check selinux if local users enabled
    repno += 1
    enforcing = commands.getstatusoutput( '/usr/sbin/getenforce' )
    if `enforcing[0]` != '0': sys.stderr.write('/usr/sbin/getenforce end with error: \'%d - %s\' !\n' % (enforcing[0], enforcing[1]))
    elif enforcing[1] == 'Enforcing': 
        ftp_home_dir = commands.getstatusoutput( '/usr/sbin/getsebool ftp_home_dir' )
        if `ftp_home_dir[0]` != '0':
            sys.stderr.write( '/usr/bin/getsebool end with error: %s - \'%s\'\n' % (ftp_home_dir[0], ftp_home_dir[1]) )
        elif ftp_home_dir[1][-3:] == 'off': 
            report(WARNING, repno, "Selinux is preventing to access user home directories." )
            report(HINT, repno, "Run: \'/usr/sbin/setsebool ftp_home_dir 1\'" )

    #: checking ports range
    repno += 1
    #report(INFO, repno, "Checking minimal value of passive port range..." )
    pasv_min_port = get_option( 'pasv_min_port', config )
    if pasv_min_port != 0 and pasv_min_port < 1024:
        report(WARNING, repno, "You shouldn't have min value of passive port range less than 1025 ! (pasv_min_port option)" )
        
    #: check for ls_recurse_enable 
    repno += 1
    ls_recurse_enable = get_option( 'ls_recurse_enable', config )
    if ls_recurse_enable == "YES":
        report(WARNING, repno, "You have \'ls_recurse_enable\' option enabled. It can cause excessive I/O on large sites !" )
    
    #: check if there can be DoS attack via ascii mangling
    repno += 1
    ascii_upload_enable = get_option( 'ascii_upload_enable', config )
    ascii_download_enable = get_option( 'ascii_download_enable', config )
    if ascii_upload_enable == "YES" or ascii_download_enable == "YES": 
        report(WARNING, repno, "Server is vulnerable to DoS attack via ascii mangling ! See hint for more information." )
        report(HINT, repno, 'By enabling ascii upload or download on some FTP servers, ASCII support allows a denial of service \
                attack (DoS) via the command \"SIZE /big/file\" in ASCII mode. \\nASCII mangling is a horrible feature of the protocol. \
                \\n\\nFor disabling use ascii_upload_enable=NO and ascii_download_enable=NO' )



if __name__ == "__main__":

    main()
