#!/usr/bin/env bash
# Copyright (C) 2008 Red Hat, Inc.
# Written by Dan Kopecek <dkopecek@redhat.com>
. "${TESTS_DIRS}/bash_defs.sh"

# --- Error IDs ------- #
const ID_AUTH_ENABLED=1
const ID_CHARGEN_ENABLED=2
const ID_ECHO_ENABLED=3
const ID_FINGER_ENABLED=4
const ID_IDENT_ENABLED=5
const ID_NETSTAT_ENABLED=6
const ID_RUSERS_ENABLED=7
const ID_SERVICE_PATHNOTABSOLUTE=8
const ID_SERVICE_PERM=9
const ID_SYSSTAT_ENABLED=10

function get_services () {
    sed -ne 's|^[[:space:]]*service[[:space:]]*\([[:alnum:]]*\)[[:space:]]*.*$|\1|p' "${1}"
}

function get_service_vars () {
    sed -ne '/^[[:space:]]*service[[:space:]]*'"${1}"'[[:space:]]*/,/^[[:space:]]*service[[:space:]]*.*$/ {
        /^[[:space:]]*[a-zA-Z_]*[[:space:]]*[-+]\{0,1\}=/,/^[[:space:]]*[a-zA-Z_]*[[:space:]]*=/ s|^[[:space:]]*\([a-zA-Z_]*\)[[:space:]]*\([-+]\{0,1\}\)=[[:space:]]*\(.*\)$|local\ \1="\3"\
local\ \1_op="\2"|p
}' "${2}"

}

function get_default_vars () {
    sed -ne '/^[[:space:]]*defaults[[:space:]]*$/,/^[[:space:]]*}[[:space:]]*$/ {
        /^[[:space:]]*[a-zA-Z_]*[[:space:]]*[-+]\{0,1\}=/,/^[[:space:]]*[a-zA-Z_]*[[:space:]]*=/ s|^[[:space:]]*\([a-zA-Z_]*\)[[:space:]]*\([-+]\{0,1\}\)=[[:space:]]*\(.*\)$|local\ \1="\3"\
local\ \1_op="\2"|p
}' "${1}"

}

function common_checks () {
    eval "${3}"
    eval "${1}"

    if [[ -n "${server}" ]]; then
	case "${server}" in
	    /*)
		check_file_perm "${server}" ${SERVICE_EXECUTABLE_PERM} ${SERVICE_EXECUTABLE_OWNER} 1 $ID_SERVICE_PERM "${2} service executable"
		;;
	    *)
		report 'ERROR' $ID_SERVICE_PATHNOTABSOLUTE "Executable path for service ${service} IS NOT ABSOLUTE."
		return ${ERR}
		;;
	esac
    fi
    
    return ${OK}
}

function check_auth () {
    eval "${2}"
    eval "${1}"

    if [[ "${disabled}" != "yes" && "${enabled}" != "no" ]] && [[ "${disable}" != "yes" ]]; then
	report 'WARNING' $ID_AUTH_ENABLED "auth service IS ENABLED"
    fi
}

function check_ident () {
    eval "${2}"
    eval "${1}"

    if [[ "${disabled}" != "yes" && "${enabled}" != "no" ]] && [[ "${disable}" != "yes" ]]; then
	report 'WARNING' $ID_IDENT_ENABLED "auth service IS ENABLED"
    fi
}

function check_systat () {
    eval "${2}"
    eval "${1}"

    if [[ "${disabled}" != "yes" && "${enabled}" != "no" ]] && [[ "${disable}" != "yes" ]]; then
	report 'WARNING' $ID_SYSSTAT_ENABLED "systat service IS ENABLED"
    fi
}

function check_chargen () {
    eval "${2}"
    eval "${1}"

    if [[ "${disabled}" != "yes" && "${enabled}" != "no" ]] && [[ "${disable}" != "yes" ]]; then
	report 'WARNING' $ID_CHARGEN_ENABLED "chargen service IS ENABLED"
    fi
}

function check_echo () {
    eval "${2}"
    eval "${1}"

    if [[ "${disabled}" != "yes" && "${enabled}" != "no" ]] && [[ "${disable}" != "yes" ]]; then
	report 'WARNING' $ID_ECHO_ENABLED "echo service IS ENABLED"
    fi
}

function check_finger () {
    eval "${2}"
    eval "${1}"

    if [[ "${disabled}" != "yes" && "${enabled}" != "no" ]] && [[ "${disable}" != "yes" ]]; then
	report 'WARNING' $ID_FINGER_ENABLED "finger service IS ENABLED"
    fi
}

function check_netstat () {
    eval "${2}"
    eval "${1}"

    if [[ "${disabled}" != "yes" && "${enabled}" != "no" ]] && [[ "${disable}" != "yes" ]]; then
	report 'WARNING' $ID_NETSTAT_ENABLED "netstat service IS ENABLED"
    fi
}

function check_rusers () {
    eval "${2}"
    eval "${1}"

    if [[ "${disabled}" != "yes" && "${enabled}" != "no" ]] && [[ "${disable}" != "yes" ]]; then
	report 'WARNING' $ID_RUSERS_ENABLED "rusers service IS ENABLED"
    fi
}

CONFIGFILES=""
DEFAULTS=""

if [[ -f "${XINETDCONF}" ]]; then
    CONFIGFILES="${XINETDCONF}"
    DEFAULTS="$(get_default_vars "${XINETDCONF}")"
fi

if [[ -d "${XINETDDIR}" ]]; then
    CONFIGFILES="$(find ${XINETDDIR} -type f) ${CONFIGFILES}"
fi

for CONF in ${CONFIGFILES}; do
    for service in $(get_services "${CONF}"); do
	VARS="$(get_service_vars "${service}" "${CONF}")"
    
	if [[ "$(type -t "check_${service}")" == "function" ]]; then
	    common_checks "${VARS}" "${service}" "${DEFAULTS}"
	    check_${service} "${VARS}" "${DEFAULTS}"
	fi
    done
done
